;************************************************************************
;*  Name      : IR_SWITCH.asm                                           *
;*  Author    : Bruce Reynolds  http://www.rentron.com                  *
;*  Date      : Feb 8th, 2006                                           *
;*  Version   : 1.0                                                     *
;*  Notes     : PIC10F200 Automatic IR Appliance Switch	                *
;*  Assembler : MPASMWIN v5.01                                          *
;*  Operation : GP0 modulates an IR LED at 40kHz by generating 'Cycles' *
;*  : number of 40kHz pulses. GP2 samples the IR detector output after  *
;*  : each carrier burst period. A logic 0 on the IR detector output    *
;*  : indicates a target is within range reflecting IR energy back onto *
;*  : the IR detector. After 6 Hits in approximately 2 seconds GP1 will *
;*  : turn on to drive a relay controlling 1 or more lights.            *
;*  : After OffTime number of samples with no Hits, approx 11 seconds,  *
;*  : the drive output on GP1 is turned back off.                       *
;*  : As long as the target is within range the output will remain on.  *
;*  : When target leaves the area, after ~11 seconds, it turns back off *
;&  : providing an auto light switch by sensing presence / no presence. *
;************************************************************************

    title       "Infrared automatic light switch"
    processor   10F200

    include     <P10F200.inc>

    __CONFIG    _IntRC_OSC & _WDT_OFF & _CP_OFF & _MCLRE_OFF

    #define IRLED GPIO,0        ; IR LED drive output pin
    #define IRDET GPIO,2        ; IR detector input pin
	
    #define LightOn bsf GPIO,1  ; turn drive output on
    #define LightOff bcf GPIO,1 ; turn drive output off

    ; OffTime = 45 = approx 11 seconds. Adjust as required
    #define OffTime D'45'   ; delay before light is turned off

    ; MAX_HITS = 6 = approx 2 seconds before light is turned on if target present
    #define MAX_HITS D'6'   ; MAX hits before light turns on

    ; setup RAM variable space
    cblock 0x10         ; starts at 0x10 for 10F200/204
      DelayTime:3       ; 3 for delay counters
      Cycles            ; for # of carrier cycles
      OffDelay          ; time counter for output drive turn off delay
      Hits              ; holds recorded # of target Hits
    endc

    org   0x00
    goto  Init          ; initialize I/O & peripherals

Main:
    movlw  D'24'        ; generate 24 x 40kHz carrier cycles (600uS burst)
    movwf  Cycles       ; load carrier cycle count
    call   Pulse        ; generate carrier burst

    movlw  D'1'         ;
    call   Delay2       ; delay between carrier bursts/target checks
    goto   Main         ; loop to Main
	
    ; infrared LED connection
    ; GPIO,0 ----\/\/\/\----|<|---- +5V DC
    ;              430     IR LED

Pulse                   ; send 'Cycles' number of 40kHz bursts
    bcf    IRLED        ; turn on IR LED here
    goto   $+1          ; 
    goto   $+1          ; 
    goto   $+1          ; 
    goto   $+1          ; 
    goto   $+1          ; 
    goto   $+1          ; 13uS with IR LED on
    bsf    IRLED        ; turn off IR LED here
    goto   $+1          ; 
    goto   $+1          ; 
    goto   $+1          ; 
    goto   $+1          ; 
    decfsz Cycles,f     ; decrement Cycles count, end pulse if Cycles = 0
    goto   Pulse        ; 25uS total = 40kHz

    btfsc  IRDET        ; test detector output. If 0, target in range, CountHits
    goto   AlarmOff     ; else jump to AlarmOff & count down turn-off period
	
CountHits
    incf   Hits,f       ; target detected, increment Hits count
    movlw  MAX_HITS     ; load MAX_HITS for comparison with current Hits count
    subwf  Hits,w       ; if Hits >= MAX_HITS then turn on drive output
    skpnc               ; if no carry, skip over LightOn, has not reached MAX_HITS
    LightOn             ; carry true, Hits = MAX_HITS, now turn on drive output
    clrf   OffDelay     ; reset OffDelay period after all Hits
    retlw  0            ; return to caller

AlarmOff
    incf   OffDelay,f   ; increment time until turn off
    movlw  OffTime      ; load OffTime for comparison with current OffDelay count
    subwf  OffDelay,w   ; if OffDelay >= OffTime, then turn drive output off
    skpnc               ; if no carry, skip over LightOff, has not reached OffTime
    LightOff            ; carry true, turn off output, OffTime period has expired
    clrf   Hits         ; reset Hits count when no target is detected
    retlw  0            ; return to caller

Delay			 
    movlw   D'2'          ; 2 = a delay period of approximately 500mS

Delay2                    ; 
    movwf   DelayTime     ; pre-load W & enter at Delay2 for different delays
    movlw   D'232'        ;
    movwf   DelayTime+1   ;
    movlw   D'255'        ;
    movwf   DelayTime+2   ;
Dloop
    clrwdt                ; 1
    decfsz  DelayTime+2,f ; 1
    goto    $-2           ; 2/1
    decfsz  DelayTime+1,f ; 1
    goto    $-4           ; 2/1
    decfsz  DelayTime,f   ; 1
    goto    $-6           ; 2/1
    retlw   0             ; 2

Init
    movwf   OSCCAL      ; load factory osccal value at start-up
    bcf     OSCCAL,0    ; do NOT output osc/4 on GP2 (IR sensor input)
    movlw   b'00000001' ; IR LED on GP0 off on power-up
    movwf   GPIO        ; load port latches
    movlw   b'00000100' ; GP2 = IR sensor input, rest outputs
    tris    GPIO        ; set I/O directions
    movlw   b'11011000'	; wake-up on pin change disabled, weak pull-ups off
                        ; Timer0 clock select on GP2 internal
    option              ; write OPTION_REG

    ; clear on boot

    movlw  0x10         ; initialize pointer
    movwf  FSR          ; to RAM start

ClearNext
    clrf   INDF         ; clear RAM location @ INDF
    incf   FSR,f        ; increment pointer to next location
    btfsc  FSR,4        ; have we cleared location 0x10 yet..?
    goto   ClearNext    ; nope, keep clearing
                        ; yep, all done, continue
	
    ; approximately ~500mS power-up delay period for
    ; stabilization time

    call   Delay
    goto   Main

    end
